from pydantic import Field

from . import ReplicaItem
from .. import _Base


class InnerContextItem(_Base):
    replicas: list[ReplicaItem] = Field(alias="Replicas")

    def to_dict(self) -> dict[str, list]:
        return self.model_dump(by_alias=True)



from pydantic import Field
from typing import Any

from . import OuterContextItem, InnerContextItem, ReplicaItemPair
from ..enums import ActionEnum
from .. import _Base


class ChatItem(_Base):
    outer_context: OuterContextItem = Field(alias="OuterContext")
    inner_context: InnerContextItem = Field(alias="InnerContext")

    def create_id(self, short: bool = False, clean: bool = False) -> str:
        return self.outer_context.create_id(short, clean)

    def to_dict(self) -> dict[str, Any]:
        return self.model_dump(by_alias=True)

    def update(self, replica_pair: ReplicaItemPair) -> None:
        self.inner_context.replicas.append(replica_pair.user_replica)
        self.inner_context.replicas.append(replica_pair.model_replica)

    def count_questions(self) -> int:
        return len(
            list(
                filter(
                    lambda replica: replica.role and replica.action == ActionEnum.QUESTION,
                    self.inner_context.replicas,
                )
            )
        )

    def zip_history(self, field: str) -> list[Any]:
        return [replica.to_dict().get(field, None) for replica in self.inner_context.replicas]



from datetime import datetime
from typing import Any, Optional

from pydantic import Field

from .. import _Base
from ..enums import StateEnum, ActionEnum, ModerationLabelEnum

_DATETIME_FORMAT: str = "%Y-%m-%d-%H-%M-%S"
_EXAMPLE_DATETIME: str = datetime(1970, 1, 1, 0, 0, 0).strftime(_DATETIME_FORMAT)


class ReplicaItem(_Base):
    body: str = Field("", alias="Body", examples=["Привет"])
    resource_id: Optional[str] = Field(None, alias="ResourceId", examples=["<link-id>"])
    role: bool = Field(False, alias="Role", description="True = ai, False = client", examples=[False])
    date_time: str = Field(
        _EXAMPLE_DATETIME,
        alias="DateTime",
        examples=[_EXAMPLE_DATETIME],
        description=f"Format: {_DATETIME_FORMAT}",
    )
    state: StateEnum = Field(
        StateEnum.EMPTY, alias="State", description="chat manager fsm state", examples=[StateEnum.COLLECTION]
    )
    action: ActionEnum = Field(
        ActionEnum.START, alias="Action", description="chat manager fsm action", examples=[ActionEnum.DIAGNOSIS]
    )
    moderation: ModerationLabelEnum = Field(
        ModerationLabelEnum.OK,
        alias="Moderation",
        description="chat manager moderated outcome type",
        examples=[ModerationLabelEnum.NON_MED],
    )

    def to_dict(self) -> dict[str, Any]:
        return self.model_dump(by_alias=True)

    @staticmethod
    def DATETIME_FORMAT() -> str:
        return _DATETIME_FORMAT


class ReplicaItemPair(_Base):
    user_replica: ReplicaItem = Field(alias="UserReplica")
    model_replica: ReplicaItem = Field(alias="ModelReplica")



from pydantic import Field
from typing import Any

from .. import _Base
from ..enums import TrackIdEnum


class OuterContextItem(_Base):
    sex: bool = Field(False, alias="Sex", description="True = male, False = female", examples=[True])
    age: int = Field(0, alias="Age", examples=[20])
    user_id: str = Field("", alias="UserId", examples=["123456789"])
    session_id: str = Field("", alias="SessionId", examples=["987654321"])
    client_id: str = Field("", alias="ClientId", examples=["543216789"])
    track_id: TrackIdEnum = Field(TrackIdEnum.CONSULTATION, alias="TrackId")

    def create_id(self, short: bool = False, clean: bool = False) -> str:
        if short:
            return f"{self.user_id}_{self.session_id}_{self.client_id}"
        if not clean:
            return f"user_{self.user_id}_session_{self.session_id}_client_{self.client_id}"
        return f"user_{self.user_id}_session_{self.session_id}_client_{self.client_id}_clean"

    def to_dict(self) -> dict[str, Any]:
        return self.model_dump(by_alias=True)



from enum import IntEnum, auto


class ModerationLabelEnum(IntEnum):
    @staticmethod
    def _generate_next_value_(name, start, count, last_values):
        return count

    OK = auto()
    NON_MED = auto()
    CHILD = auto()
    ABSURD = auto()
    GREETING = auto()
    RECEIPT = auto()



from typing import Self

from pydantic import BaseModel, model_validator, SecretStr


class GigaChatConfig(BaseModel):

    client_id: SecretStr = SecretStr("")
    client_secret: SecretStr = SecretStr("")

    @model_validator(mode="after")
    def empty_validator(self) -> Self:
        if not (self.client_id and self.client_secret):
            raise ValueError("creds for gigachat is not filled")
        return self
